@@ -1,10 +1,22 @@
 package CommonTest;
 
 use v5.16.3;
+use Archive::Tar;
+use File::chdir;
 use Test::Roo::Role;
 use Test::TempDir::Tiny;
 use Path::Tiny;
-use IPC::Cmd qw/can_run run run_forked/;
+use IPC::Cmd 0.92 qw/can_run run run_forked/;
+
+has _openssl => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $p = can_run 'openssl';
+        BAIL_OUT "openssl not found, cannot continue" unless $p;
+        return path $p;
+    },
+);
 
 has workdir => (
     is      => 'ro',
@@ -14,18 +26,13 @@ has workdir => (
 );
 
 has hsm_token_dir => (
-    is      => 'ro',
+    is      => 'lazy',
     clearer => 1,
-    lazy    => 1,
-    default => sub {
-        my $dir = $_[0]->workdir->child('tokens');
-        $dir->mkpath;
-        return $dir;
-    },
 );
 
 has hsm_config => (
     is      => 'ro',
+    lazy    => 1,
     default => sub {
         sprintf "directories.tokendir = %s\nobjectstore.backend = file",
           $_[0]->hsm_token_dir;
@@ -51,46 +58,32 @@ has key => (
 );
 
 has has_softhsm2 => (
-    is      => 'ro',
-    lazy    => 1,
-    default => sub {
-        return can_run('softhsm2-util');
-    },
-);
-
-has _softhsm_util => (
     is      => 'ro',
     lazy    => 1,
     default => sub {
         my $p = can_run 'softhsm2-util';
-        BAIL_OUT "softhsm2-util not found, cannot continue" unless $p;
-        return path $p;
+        return $p ? path $p : undef;
     },
 );
 
-has _openssl => (
-    is      => 'ro',
-    lazy    => 1,
-    default => sub {
-        my $p = can_run 'openssl';
-        BAIL_OUT "openssl not found, cannot continue" unless $p;
-        return path $p;
-    },
-);
-
-sub BUILD {
-
-    if ($ENV{TEST_DEBUG}) {
-        require Log::Any::Adapter;
-        Log::Any::Adapter->set('Stderr');
-    }
-
-    return;
-}
-
 has pkcs11 =>
   (is => 'rw', lazy => 1, builder => '_build_pkcs11', clearer => 1);
 
+before setup => sub {
+    my $self = shift;
+
+    if ($ENV{TEST_DEBUG}) {
+        require Log::Any::Adapter;
+        Log::Any::Adapter->set('Stderr', log_level => 'debug');
+    }
+};
+
+after each_test => sub {
+    my $self = shift;
+    $self->clear_pkcs11;
+    $self->clear_hsm_token_dir;
+};
+
 sub _build_pkcs11 {
     my $self = shift;
     my $mod  = 'Crypt::PKCS11::Easy';
@@ -107,7 +100,8 @@ sub _build_pkcs11 {
 }
 
 sub _new_pkcs11 {
-    my ($self, $key, $slot, $func) = @_;
+    my $self = shift;
+    my %args = @_;
 
     my $mod = 'Crypt::PKCS11::Easy';
 
@@ -117,9 +111,10 @@ sub _new_pkcs11 {
 
     my $args = [module => 'libsofthsm2'];
 
-    push @$args, key      => $key  if $key;
-    push @$args, slot     => $slot if defined $slot;
-    push @$args, function => $func if defined $func;
+    push @$args, key      => $args{key}  if $args{key};
+    push @$args, slot     => $args{slot} if defined $args{slot};
+    push @$args, function => $args{func} if $args{func};
+    push @$args, mech     => $args{mech} if $args{mech};
     push @$args, pin      => '1234';
 
     my $obj = new_ok $mod => $args;
@@ -129,41 +124,17 @@ sub _new_pkcs11 {
     return $obj;
 }
 
-after each_test => sub { shift->clear_pkcs11 };
+sub _build_hsm_token_dir {
+    my $self = shift;
 
-sub init_token {
-    my ($self, $slot, $label) = @_;
-    my @cmd = (
-        $self->_softhsm_util, '--init-token', '--pin', '1234', '--so-pin',
-        '123456', '--slot', $slot, '--label', $label
-    );
+    my $archive = path('t/data/tokens.tar.gz')->absolute;
 
-    run
-      command => \@cmd,
-      verbose => $ENV{TEST_DEBUG},
-      timeout => 10
-      or die "Failed to initialise token";
+    local $CWD = $self->workdir;
 
-    return;
-}
+    diag "Extracting test tokens into $CWD";
+    Archive::Tar->extract_archive($archive);
 
-sub import_key {
-    my ($self, $slot, $key, $label) = @_;
-
-    $key = path 't', 'keys', "$key.pkcs8";
-
-    my @cmd = (
-        $self->_softhsm_util, '--pin', '1234', '--slot', $slot, '--import',
-        $key, '--label', $label, '--id', '0000'
-    );
-
-    run
-      command => \@cmd,
-      verbose => $ENV{TEST_DEBUG},
-      timeout => 10
-      or die "Failed to initialise token";
-
-    return;
+    return path('tokens')->absolute;
 }
 
 sub openssl_sign {
@@ -175,4 +146,36 @@ sub openssl_sign {
     return $output->{stdout};
 }
 
+sub openssl_verify {
+    my ($self, $key_file, $sig_file, $data_file) = @_;
+
+    my $openssl_cmd = [
+        $self->_openssl, 'dgst',       '-sha1',   '-verify',
+        $key_file,       '-signature', $sig_file, $data_file
+    ];
+
+    return run command => $openssl_cmd, verbose => $ENV{TEST_DEBUG};
+}
+
+sub openssl_decrypt {
+    my ($self, $private_key_file, $encrypted_data, $mech) = @_;
+
+    my $openssl_cmd =
+      [$self->_openssl, 'rsautl', '-decrypt', '-inkey', $private_key_file];
+
+    if ($mech eq 'RSA_PKCS') {
+        push @$openssl_cmd, '-pkcs';
+    } elsif ($mech eq 'RSA_PKCS_OAEP') {
+        push @$openssl_cmd, '-oaep';
+    } else {
+        die "Unsupported mech: $mech";
+    }
+
+    my $output = run_forked $openssl_cmd,
+      {verbose => $ENV{TEST_DEBUG}, child_stdin => $encrypted_data};
+    chomp $output->{stdout};
+
+    return $output->{stdout};
+}
+
 1;
